@@ -284,5 +284,5 @@ EB <- c("133746","1339","134","1340","134083","134111","134147","134187","134218
 "1370","137075","1371","137209","1373","137362","1374","137492","1375","1376",
 "137682","137695","137735","1378","137814","137868","137872","137886","137902","137964")
 go <- goana(fit,FDR=0.8,geneid=EB)
-topGO(go,n=10)
-topGO(go,n=10,sort="down")
+topGO(go,n=10,truncate.term=30)
+topGO(go,n=10,truncate.term=30,sort="down")
